import { useParams, Link } from "react-router-dom";
import React, { useEffect, useState } from "react";
import { Channel_type, Messages } from "../types";
import axios from "axios";

export default function Channel() {
    const { name } = useParams();
    const [channel, setChannel] = useState<Channel_type>();
    const [messages, setMessages] = useState<Messages>();
    const [token, setToken] = useState<string>("");
    const [message, setMessage] = useState<string>("");
    const [maxMessageToShown, setMaxMessageToShown] = useState<number>(10);

    function handleSubmit(e: React.FormEvent) {
        e.preventDefault();
        axios
            .post(`/api/channels/${name}/messages/send`, { token, message})
            .then(() => {
                axios
                    .get(`/api/channels/${name}/messages`)
                    .then((res) => {
                        setMessages(res.data);
                        setMessage("");
                    }
                );
            })
            .catch((err) => {
                console.error(err.response.data.message);
            });
    }

    useEffect(() => {
        const localToken = localStorage.getItem("token");
        if (localToken) setToken(localToken);
        axios.get(`/api/channels/${name}`).then((res) => {
            setChannel(res.data);
        });

        axios.get(`/api/channels/${name}/messages`).then((res) => {
            setMessages(res.data);
        });
    }, [name]);

    if (!channel || !messages) {
        return <div>Loading...</div>;
    }

    return (
        <div>
            <Link to="/">Home</Link>
            <h1>Channel {channel.name}</h1>
            <p>{channel.description}</p>
            {
                token ? (
                    <form onSubmit={handleSubmit}>
                        <input 
                            type="text"
                            placeholder="Message"
                            value={message}
                            onChange={(e) => setMessage(e.target.value)}
                        />
                        <button type="submit">Send</button>
                    </form>
                ) : (
                    <div>
                        <Link to="/login">Login</Link>
                        <Link to="/register">Register</Link>
                    </div>
                )
            }
            <ul>
                {messages.slice(0, maxMessageToShown).map((message) => (
                    <li key={message.id}>
                        <p>{message.username}: {message.content}</p>
                    </li>
                ))}
            </ul>
            {
                messages.length > maxMessageToShown && (
                    <button onClick={() => setMaxMessageToShown(maxMessageToShown + 10)}>Show more</button>
                )
            }
        </div>
    );
}